package com.example.ueberwachungssystem;

import android.content.Context;
import android.graphics.ImageFormat;
import android.media.Image;
import android.util.Log;
import android.util.Size;

import androidx.annotation.NonNull;
import androidx.camera.core.CameraSelector;
import androidx.camera.core.ExperimentalGetImage;
import androidx.camera.core.ImageAnalysis;
import androidx.camera.core.ImageProxy;
import androidx.camera.core.Preview;
import androidx.camera.lifecycle.ProcessCameraProvider;
import androidx.core.content.ContextCompat;
import androidx.lifecycle.LifecycleOwner;

import com.google.common.util.concurrent.ListenableFuture;

import java.nio.ByteBuffer;
import java.util.concurrent.ExecutionException;


@ExperimentalGetImage
public class VideoDetector {

    private final Context context;
    private float currentLuminosity;


    public VideoDetector(Context context) {
        this.context = context;
    }

    public void runImageAnalysis() {
        // Request Camera Provider
        final ListenableFuture<ProcessCameraProvider> cameraProviderFuture = ProcessCameraProvider.getInstance(context);
        //Check for Camera availability
        cameraProviderFuture.addListener(new Runnable() {
            @Override
            public void run() {
                try {
                    ProcessCameraProvider cameraProvider = cameraProviderFuture.get();
                    bindImageAnalysis(cameraProvider);
                } catch (ExecutionException | InterruptedException e) {
                    // No errors need to be handled for this Future. This should never be reached.
                }
            }
        },ContextCompat.getMainExecutor(context));
    }

    private void bindImageAnalysis(@NonNull ProcessCameraProvider cameraProvider) {
        // Configure and create Image Analysis
        ImageAnalysis.Builder builder = new ImageAnalysis.Builder();
        builder.setTargetResolution(new Size(640, 480));
        builder.setBackpressureStrategy(ImageAnalysis.STRATEGY_KEEP_ONLY_LATEST);
        ImageAnalysis imageAnalysis = builder.build();

        // Set Analyzer
        imageAnalysis.setAnalyzer(ContextCompat.getMainExecutor(context), new ImageAnalysis.Analyzer() {
            @Override
            public void analyze(@NonNull ImageProxy imageProxy) {
                if (imageProxy.getFormat() == ImageFormat.YUV_420_888) {
                    Image image = imageProxy.getImage();
                    assert image != null;

                    float luminosity = calculateLuminosity(image);
                    currentLuminosity = luminosity;
                    Log.d("Video Detector", String.valueOf(luminosity));
                }
                imageProxy.close();
            }
        });
        // Create Preview
        Preview preview = new Preview.Builder().build();
        // Specify which Camera to use
        CameraSelector cameraSelector = new CameraSelector.Builder().requireLensFacing(CameraSelector.LENS_FACING_BACK).build();
        // Connect Preview to PreviewView
        cameraProvider.bindToLifecycle((LifecycleOwner) context, cameraSelector, imageAnalysis, preview);
    }


    private float calculateLuminosity (Image image) {
        int width = image.getWidth();
        int height = image.getHeight();

        Image.Plane[] planes = image.getPlanes();
        ByteBuffer luminosityBuffer = planes[0].getBuffer();

        int yRowStride = image.getPlanes()[0].getRowStride();
        int yPixelStride = image.getPlanes()[0].getPixelStride();

        int luminosity;
        float sum = 0;

        for (int y = 0; y < height; ++y) {
            for (int x = 0; x < width; x++) {
                int index = (y * yRowStride) + (x * yPixelStride);
                luminosity = (luminosityBuffer.get(index) & 0xff);
                sum += luminosity;
            }
        }
        return sum / (width * height);
    }
}
